// Code generated by MockGen. DO NOT EDIT.
// Source: gitlab.com/gitlab-org/cli/internal/cmdutils (interfaces: Executor)
//
// Generated by this command:
//
//	mockgen -typed -destination=./executor_mocks.go -package=cmdtest gitlab.com/gitlab-org/cli/internal/cmdutils Executor
//

// Package cmdtest is a generated GoMock package.
package cmdtest

import (
	context "context"
	io "io"
	reflect "reflect"

	gomock "go.uber.org/mock/gomock"
)

// MockExecutor is a mock of Executor interface.
type MockExecutor struct {
	ctrl     *gomock.Controller
	recorder *MockExecutorMockRecorder
	isgomock struct{}
}

// MockExecutorMockRecorder is the mock recorder for MockExecutor.
type MockExecutorMockRecorder struct {
	mock *MockExecutor
}

// NewMockExecutor creates a new mock instance.
func NewMockExecutor(ctrl *gomock.Controller) *MockExecutor {
	mock := &MockExecutor{ctrl: ctrl}
	mock.recorder = &MockExecutorMockRecorder{mock}
	return mock
}

// EXPECT returns an object that allows the caller to indicate expected use.
func (m *MockExecutor) EXPECT() *MockExecutorMockRecorder {
	return m.recorder
}

// Exec mocks base method.
func (m *MockExecutor) Exec(ctx context.Context, name string, args, env []string) error {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Exec", ctx, name, args, env)
	ret0, _ := ret[0].(error)
	return ret0
}

// Exec indicates an expected call of Exec.
func (mr *MockExecutorMockRecorder) Exec(ctx, name, args, env any) *MockExecutorExecCall {
	mr.mock.ctrl.T.Helper()
	call := mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Exec", reflect.TypeOf((*MockExecutor)(nil).Exec), ctx, name, args, env)
	return &MockExecutorExecCall{Call: call}
}

// MockExecutorExecCall wrap *gomock.Call
type MockExecutorExecCall struct {
	*gomock.Call
}

// Return rewrite *gomock.Call.Return
func (c *MockExecutorExecCall) Return(arg0 error) *MockExecutorExecCall {
	c.Call = c.Call.Return(arg0)
	return c
}

// Do rewrite *gomock.Call.Do
func (c *MockExecutorExecCall) Do(f func(context.Context, string, []string, []string) error) *MockExecutorExecCall {
	c.Call = c.Call.Do(f)
	return c
}

// DoAndReturn rewrite *gomock.Call.DoAndReturn
func (c *MockExecutorExecCall) DoAndReturn(f func(context.Context, string, []string, []string) error) *MockExecutorExecCall {
	c.Call = c.Call.DoAndReturn(f)
	return c
}

// ExecWithCombinedOutput mocks base method.
func (m *MockExecutor) ExecWithCombinedOutput(ctx context.Context, name string, args, env []string) ([]byte, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "ExecWithCombinedOutput", ctx, name, args, env)
	ret0, _ := ret[0].([]byte)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// ExecWithCombinedOutput indicates an expected call of ExecWithCombinedOutput.
func (mr *MockExecutorMockRecorder) ExecWithCombinedOutput(ctx, name, args, env any) *MockExecutorExecWithCombinedOutputCall {
	mr.mock.ctrl.T.Helper()
	call := mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "ExecWithCombinedOutput", reflect.TypeOf((*MockExecutor)(nil).ExecWithCombinedOutput), ctx, name, args, env)
	return &MockExecutorExecWithCombinedOutputCall{Call: call}
}

// MockExecutorExecWithCombinedOutputCall wrap *gomock.Call
type MockExecutorExecWithCombinedOutputCall struct {
	*gomock.Call
}

// Return rewrite *gomock.Call.Return
func (c *MockExecutorExecWithCombinedOutputCall) Return(arg0 []byte, arg1 error) *MockExecutorExecWithCombinedOutputCall {
	c.Call = c.Call.Return(arg0, arg1)
	return c
}

// Do rewrite *gomock.Call.Do
func (c *MockExecutorExecWithCombinedOutputCall) Do(f func(context.Context, string, []string, []string) ([]byte, error)) *MockExecutorExecWithCombinedOutputCall {
	c.Call = c.Call.Do(f)
	return c
}

// DoAndReturn rewrite *gomock.Call.DoAndReturn
func (c *MockExecutorExecWithCombinedOutputCall) DoAndReturn(f func(context.Context, string, []string, []string) ([]byte, error)) *MockExecutorExecWithCombinedOutputCall {
	c.Call = c.Call.DoAndReturn(f)
	return c
}

// ExecWithIO mocks base method.
func (m *MockExecutor) ExecWithIO(ctx context.Context, name string, args, env []string, stdin io.Reader, stdout, stderr io.Writer) error {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "ExecWithIO", ctx, name, args, env, stdin, stdout, stderr)
	ret0, _ := ret[0].(error)
	return ret0
}

// ExecWithIO indicates an expected call of ExecWithIO.
func (mr *MockExecutorMockRecorder) ExecWithIO(ctx, name, args, env, stdin, stdout, stderr any) *MockExecutorExecWithIOCall {
	mr.mock.ctrl.T.Helper()
	call := mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "ExecWithIO", reflect.TypeOf((*MockExecutor)(nil).ExecWithIO), ctx, name, args, env, stdin, stdout, stderr)
	return &MockExecutorExecWithIOCall{Call: call}
}

// MockExecutorExecWithIOCall wrap *gomock.Call
type MockExecutorExecWithIOCall struct {
	*gomock.Call
}

// Return rewrite *gomock.Call.Return
func (c *MockExecutorExecWithIOCall) Return(arg0 error) *MockExecutorExecWithIOCall {
	c.Call = c.Call.Return(arg0)
	return c
}

// Do rewrite *gomock.Call.Do
func (c *MockExecutorExecWithIOCall) Do(f func(context.Context, string, []string, []string, io.Reader, io.Writer, io.Writer) error) *MockExecutorExecWithIOCall {
	c.Call = c.Call.Do(f)
	return c
}

// DoAndReturn rewrite *gomock.Call.DoAndReturn
func (c *MockExecutorExecWithIOCall) DoAndReturn(f func(context.Context, string, []string, []string, io.Reader, io.Writer, io.Writer) error) *MockExecutorExecWithIOCall {
	c.Call = c.Call.DoAndReturn(f)
	return c
}

// LookPath mocks base method.
func (m *MockExecutor) LookPath(file string) (string, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "LookPath", file)
	ret0, _ := ret[0].(string)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// LookPath indicates an expected call of LookPath.
func (mr *MockExecutorMockRecorder) LookPath(file any) *MockExecutorLookPathCall {
	mr.mock.ctrl.T.Helper()
	call := mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "LookPath", reflect.TypeOf((*MockExecutor)(nil).LookPath), file)
	return &MockExecutorLookPathCall{Call: call}
}

// MockExecutorLookPathCall wrap *gomock.Call
type MockExecutorLookPathCall struct {
	*gomock.Call
}

// Return rewrite *gomock.Call.Return
func (c *MockExecutorLookPathCall) Return(arg0 string, arg1 error) *MockExecutorLookPathCall {
	c.Call = c.Call.Return(arg0, arg1)
	return c
}

// Do rewrite *gomock.Call.Do
func (c *MockExecutorLookPathCall) Do(f func(string) (string, error)) *MockExecutorLookPathCall {
	c.Call = c.Call.Do(f)
	return c
}

// DoAndReturn rewrite *gomock.Call.DoAndReturn
func (c *MockExecutorLookPathCall) DoAndReturn(f func(string) (string, error)) *MockExecutorLookPathCall {
	c.Call = c.Call.DoAndReturn(f)
	return c
}
