// Code generated by MockGen. DO NOT EDIT.
// Source: golang.org/x/oauth2 (interfaces: TokenSource)
//
// Generated by this command:
//
//	mockgen -typed -destination=./mocks_for_test.go -package=credentialhelper golang.org/x/oauth2 TokenSource
//

// Package credentialhelper is a generated GoMock package.
package credentialhelper

import (
	reflect "reflect"

	gomock "go.uber.org/mock/gomock"
	oauth2 "golang.org/x/oauth2"
)

// MockTokenSource is a mock of TokenSource interface.
type MockTokenSource struct {
	ctrl     *gomock.Controller
	recorder *MockTokenSourceMockRecorder
	isgomock struct{}
}

// MockTokenSourceMockRecorder is the mock recorder for MockTokenSource.
type MockTokenSourceMockRecorder struct {
	mock *MockTokenSource
}

// NewMockTokenSource creates a new mock instance.
func NewMockTokenSource(ctrl *gomock.Controller) *MockTokenSource {
	mock := &MockTokenSource{ctrl: ctrl}
	mock.recorder = &MockTokenSourceMockRecorder{mock}
	return mock
}

// EXPECT returns an object that allows the caller to indicate expected use.
func (m *MockTokenSource) EXPECT() *MockTokenSourceMockRecorder {
	return m.recorder
}

// Token mocks base method.
func (m *MockTokenSource) Token() (*oauth2.Token, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Token")
	ret0, _ := ret[0].(*oauth2.Token)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// Token indicates an expected call of Token.
func (mr *MockTokenSourceMockRecorder) Token() *MockTokenSourceTokenCall {
	mr.mock.ctrl.T.Helper()
	call := mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Token", reflect.TypeOf((*MockTokenSource)(nil).Token))
	return &MockTokenSourceTokenCall{Call: call}
}

// MockTokenSourceTokenCall wrap *gomock.Call
type MockTokenSourceTokenCall struct {
	*gomock.Call
}

// Return rewrite *gomock.Call.Return
func (c *MockTokenSourceTokenCall) Return(arg0 *oauth2.Token, arg1 error) *MockTokenSourceTokenCall {
	c.Call = c.Call.Return(arg0, arg1)
	return c
}

// Do rewrite *gomock.Call.Do
func (c *MockTokenSourceTokenCall) Do(f func() (*oauth2.Token, error)) *MockTokenSourceTokenCall {
	c.Call = c.Call.Do(f)
	return c
}

// DoAndReturn rewrite *gomock.Call.DoAndReturn
func (c *MockTokenSourceTokenCall) DoAndReturn(f func() (*oauth2.Token, error)) *MockTokenSourceTokenCall {
	c.Call = c.Call.DoAndReturn(f)
	return c
}
