package login

// Prompt option constants for login flow
const (
	// Host selection options
	promptLoginDifferentHostname         = "Enter a different hostname"
	promptSelfManagedOrDedicatedInstance = "GitLab Self-Managed or GitLab Dedicated instance"

	// Login type options
	promptLoginTypeToken  = "Token"
	promptLoginTypeWeb    = "Web"
	promptLoginTypeDevice = "Device"

	// Protocol options
	promptProtocolSSH   = "SSH"
	promptProtocolHTTPS = "HTTPS"
	promptProtocolHTTP  = "HTTP"
)
